# -*- coding: utf-8 -*-
"""
Created on Sat May 05 08:29:35 2012

@author: Owner
"""

#import python modules
import numpy as np
import ScottShell as SS
import itertools
import time
import pickle
import atomwrite
import matplotlib.pyplot as plt

#import compiled fortran libraries
import finallib

#define functions::
    
#import sys
import os

#NOTE:
#everything below assumes unit atomic masses,
#such that forces = accelerations.


#========== GLOBAL VARIABLES ==========
#none

#Set flags to do different parts of the assignment
Do_CG = False
PrintImages = True
DoMixing = False
DoPdb = False

A = 1 #number of A atoms in single copolymer
B = 1 #number of B atoms in single copolymer
Np = 130 #number of copolymers
N = Np*(A+B) #total number of atoms
M = A + B #total number of atoms in single copolymer
rhoList = [0.1,0.5,0.01,0.05] #list of densities to iterate over
TList = [2.0,1.0,0.5,0.1] #list of temperatures to iterate over
LambdaList = [0.0,0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0,2.5,
              3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5] #list of lambdas to iterate over
Nrho = len(rhoList)
NT = len(TList)
NLambda = len(LambdaList)
Type,Names = SS.SetTypes(N,A,B)
if PrintImages and DoPdb:
    for i in range(7):
        Names.extend(Names[0:N])
Cut = 4.0 #cutoff distance for potential interaction
dt = 0.001 #Timestep size
RescaleFreq = 100 #Number of steps between rescaling the velocity
NStepsMix = int(3e2)
NStepsEquil = int(3e6)
NStepsProd = int(2e4)
StructureFreq = 100
EquilFreq = 1000
EquilTol = 1e-3
PdbFreq = 100
Bins = 1000

dx = 0.001 #Initial step size for linesearch
EFracTolLS = 1e-8 #Error tolerance used in Line search
if DoMixing:
    EFracTolCG = 1e-4 #Error tolerance used in conjugate gradient
else:
    EFracTolCG = 1e-6 #Error tolerance used in conjugate gradient
Colors = ['k','g','r','b','m','k','g','r','b']
Symbols = ['.','^','s','p','h','^','s','p','h']
Lines = ['--','-','-.',':']
Entry = []

#Check for bad parameter settings
if (N/max(rhoList))**(1./3.)/2. < Cut:
    raise Exception, "Not enough molecules to make a big enough box!"


#Initialize storage variables




#loop simulation over all combinations of rho and M the number of times 
#specified in Trials.  This is the equivalent of three nested loops.
t1t = time.time()


for (i,j,k) in itertools.product(range(Nrho),range(NT),range(NLambda)):
    
    #Set variable values
    plt.close('all')
    print (i,j,k)
    rho = rhoList[i]
    T = TList[j]
    Lambda = LambdaList[k]
    L = (N/rho)**(1./3.)
    Tinitial = np.max([1.0,2.0*T])
    PdbFreq = int(100/T)
    
    
    
    t1a = time.time()
    CG_Output = 'rho=%.2g T=%.2g N=%i Lambda=%.3g Cut=%.1f.p' \
                % (rho,T,N,Lambda,Cut)
                
    if os.path.exists(CG_Output + 'kl') or \
        os.path.exists('Start' + CG_Output + 'kl'):
        print 'Skipping because data already generated.'        
        continue
    
    pickle.dump(L, open('Start' + CG_Output + 'kl', 'wb'), -1)
    
    if not Do_CG and os.path.exists(CG_Output):
        Pos = pickle.load(open(CG_Output,'rb'))
        PE = finallib.calcenergy(Pos,Type,M,L,Cut,Lambda)
    else:
        #Initialize the atoms on a cubic lattice
#        Pos = SS.InitPositions(N,L)
        Pos = (N)**(1./3.)*np.random.rand(N, 3)
        #Use the conjugate gradient method to find an equlibrium configuration
        PE, Pos = SS.ConjugateGradient(Pos,dx,Type,M,L,Cut,Lambda,
                                       EFracTolLS, EFracTolCG)
        #Write to file
        pickle.dump(Pos,open(CG_Output,'wb'))
    
    t2a = time.time()
    t3a = t2a-t1a
    print "The Conjugate Gradient time was %.2f sec" % t3a
    
    
    
    #Initialize the velocity and accelaration variables
    if not DoMixing:
        Tinitial = T
    Vel = SS.InitVelocities(N,Tinitial)
    Accel = SS.InitAccel(Pos,Type,M,L,Cut,Lambda)

    
    
    #Do mixing run to randomize positions without overstretching bonds
    if DoMixing:
        t1a = time.time()
        U = np.zeros(NStepsMix)
#        EquilTol /= 10.
        
        for ii in range(NStepsMix):
    #        #Rescale velocity every RescaleFreq steps
    #        if ii % RescaleFreq == 0:
    #            Vel = SS.RescaleVelocities(Vel, T)
            #Use verlet alogrithm for time integration
            Pos,Vel,Accel,KE,PE = finallib.vvintegrate(Pos,Vel,Accel,Type,M,L,
                                                       Cut,Lambda,dt)
            #Store energy at each time step
            U[ii] = PE
    
        #plot data
        PlotName = 'Mix(%.2g,%.2g,%.2g).png' % (rho,T,Lambda)
        plt.figure(PlotName)
        plt.plot(U,'b')
        plt.xlabel('Time Steps')
        plt.ylabel('Potential Energy')
        plt.savefig(PlotName)
        
        del U
        
        t2a = time.time()
        t3a = t2a-t1a
        print "The mixing time for (%.2g,%.2g,%.2g) was %.2f sec" \
        % (rho,T,Lambda,t3a)
    
    

    #Do equilibration run
    if DoPdb:
        Pdb = atomwrite.pdbfile(CG_Output + 'db',3.*L,AtomNames = Names)
    t1a = time.time()
    U = np.zeros(NStepsEquil)
    Utest = np.zeros(EquilFreq)
    change = 2.*EquilTol
    PEold = PE
    EquilFreq1 = EquilFreq
    CheckPoint = 10*EquilFreq
    for ii in range(NStepsEquil):
        #Rescale velocity every RescaleFreq steps
        if ii % RescaleFreq == 0:
            Vel = SS.RescaleVelocities(Vel, T)
        #Save positions to pdb file
        if ii % PdbFreq == 0 and DoPdb:
            Pdb.write(SS.MinImage(Pos,L,PrintImages))
        #Use verlet alogrithm for time integration
        Pos,Vel,Accel,KE,PE = finallib.vvintegrate(Pos,Vel,Accel,Type,M,L,
                                                   Cut,Lambda,dt)
        #Store energy at each time step
        U[ii] = PE
        Utest[ii % EquilFreq1] = PE
        #Do periodic checking for equilibration
        if (EquilFreq1 - (ii % EquilFreq1)) == 1:
            PEnew = np.mean(Utest)
            change = np.abs((PEnew - PEold)/PEnew)
            if np.abs((PEnew - PEold)) < T/2.0 and PEnew > T:
                change = 0.0
            print 'Current PE = %.4g change = %.4g' % (PEnew,change)
            PEold = PEnew
        if ii == CheckPoint:
            print '%i iterations performed' % CheckPoint
            CheckPoint *= 2            
            EquilFreq1 *= 2
            Utest = np.append(Utest,np.copy(Utest))
        if change < EquilTol and ii > 10*EquilFreq:
            break
            
            
    
    #Truncate array at equilibrated value
    U = U[0:ii]
    
    #plot data
    PlotName = 'Equilibration(%.2g,%.2g,%.2g).png' % (rho,T,Lambda)
    plt.figure(PlotName)
    plt.plot(U,'b')
    plt.xlabel('Time Steps')
    plt.ylabel('Potential Energy')
    plt.savefig(PlotName) 
    
    del U
    
    t2a = time.time()
    t3a = t2a-t1a
    print "The equilibration time for (%.2g,%.2g,%.2g) was %.2f sec" \
    % (rho,T,Lambda,t3a) 
    t1a = time.time()
    
    
    
    #Do a production run to get structure statistics
    U = np.zeros(NStepsProd + 3*EquilFreq1)
    cRDF = np.zeros(Bins)
    cRDFAA = np.zeros_like(cRDF)
    cRDFAB = np.zeros_like(cRDF)
    cN = np.zeros(N)
    count = 0
    for ii in range(NStepsProd + 3*EquilFreq1):
        #Rescale velocity every RescaleFreq steps
        if ii % RescaleFreq == 0:
            Vel = SS.RescaleVelocities(Vel, T)
        if ii % PdbFreq == 0 and DoPdb:
            Pdb.write(SS.MinImage(Pos,L,PrintImages))
        if ii % StructureFreq == 0:
            cN,cRDF,cRDFAA,cRDFAB = \
                finallib.structure(Pos,Type,M,L,Lambda,cN,cRDF,cRDFAA,cRDFAB)
            count += 1
            
        #Use verlet alogrithm for time integration\
        Pos,Vel,Accel,KE,PE = finallib.vvintegrate(Pos,Vel,Accel,Type,M,L,
                                                   Cut,Lambda,dt)
        #Store energy at each time step
        U[ii] = PE
        
    if DoPdb:
        Pdb.close()
    
    #plot data
    PlotName = 'Production(%.2g,%.2g,%.2g).png' % (rho,T,Lambda)
    plt.figure(PlotName)
    plt.plot(U,'b')
    plt.xlabel('Time Steps')
    plt.ylabel('Potential Energy')
    plt.savefig(PlotName)

    #plot data
    PlotName = 'RDF(%.2g,%.2g,%.2g).png' % (rho,T,Lambda)
    plt.figure(PlotName)
    width = L/2.0/Bins
    edges = np.arange(Bins+1)*width
    V = L**3.
    dV = 4./3.*np.pi*(edges[1:]**3. - edges[:-1]**3.)
    RDF = cRDF/count/(N)/dV/rho    
    RDFAA = cRDFAA/count/(Np*A)/dV/(Np*A-1)*V/2.
    RDFAB = cRDFAB/count/(Np*A)/dV/(Np*B)*V/2.
    r = (np.arange(Bins)+0.5)*width
    plt.subplot(2,1,1)
    plt.plot(r,RDF,'k-',r,RDFAA,'r-',r,RDFAB,'g-',r,np.ones_like(r),'b--',
             lw=2.0)
    maxY = 1.1*max([2.0,max(RDFAB[1.1/(L/2.)*Bins:])])
    plt.ylim((0.0,maxY))
    maxX = min([np.max(r),8.0])
    plt.xlim((0.0,maxX))
    plt.xlabel('r')
    plt.ylabel('RDF')
    
    plt.subplot(2,1,2)
    plt.plot(r,RDF,'k-',r,RDFAA,'r-',r,RDFAB,'g-',r,np.ones_like(r),'b--',
             lw=2.0)
    plt.xlim((0.95,1.25))
    plt.xlabel('r')
    plt.ylabel('RDF')
    plt.legend(('overall','AA','AB'),numpoints=1)
    plt.savefig(PlotName)

    #plot data
    PlotName = 'cN(%.2g,%.2g,%.2g).png' % (rho,T,Lambda)
    plt.figure(PlotName)
    cN = cN/count
    xmax = min([2.0*np.max(cN),16])
    cNrange = np.max(cN) - np.min(cN)
    cNbins = max([int(cNrange/.25),8])
    plt.hist(cN,bins=cNbins,range=(0,xmax))
    plt.xlabel('Coordination Number')
    plt.ylabel('Number of Atoms')
    plt.savefig(PlotName)
    
    del U
    
    t2a = time.time()
    t3a = t2a-t1a
    print "The production time for (%.2g,%.2g,%.2g) was %.2f sec" \
    % (rho,T,Lambda,t3a) 
    
    #write data as a dictionary for pickling
    data = {'cN' : cN, 'RDF' : RDF, 'RDFAA' : RDFAA, 'cRDFAB' : cRDFAB,
            'count': count, 'L': L, 'r': r, 'EquilFreq1' : EquilFreq1}
    pickle.dump(data, open(CG_Output + 'kl', 'wb'), -1)
    
    
    
t2t = time.time()
t3t = t2t - t1t
print "The total time required was %.2f sec" % (t3t)