subroutine CalcEnergy(Pos, Type, M, L, rc, Lambda, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, Lambda
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.0
    integer :: i,j !Local
    real(8) :: d, d2, id2, id4, id6, id12, rc2, Half_k, Shift, RepelShift, Typei
    real(8), dimension(Dim) :: rij, Posi

    !compute shift energy for cutoff distance of rc
    PEnergy = 0.0
    rc2 = rc*rc
    Shift = -4.0*(rc**(-12) - rc**(-6))
    RepelShift = -4.0*(rc**(-12) + Lambda*rc**(-4))
    Half_k = 0.5*k

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1

            rij = Pos(j,:) - Posi

            !Convert to minimum image distance for periodic boundaries
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
            !Compute the energy and force depending on whether the atoms are 
            !bonded or not.
            if (j == i+1 .and. mod(j,M) > 0) then !bonded
                d = sqrt(d2)
                PEnergy = PEnergy + Half_k*(d - r0)*(d - r0)
            !check to see if we are within the cutuff distance
            elseif (d2 < rc2) then !not bonded
                if ((Typei - Type(j)) == 0) then !same type of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id6 = id2*id2*id2    !inverse sixth distance
                    id12 = id6*id6         !inverse twelfth distance
                    PEnergy = PEnergy + 4.*(id12 - id6) + Shift
                else !different kind of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id4 = id2*id2    !inverse fourth distance
                    id12 = id4*id4*id4        !inverse twelfth distance
                    PEnergy = PEnergy + 4.*(id12 + Lambda*id4) + RepelShift
                endif ! atom type test
            endif ! bonded test
        enddo
    enddo
end subroutine


subroutine CalcEnergyVirial(Pos, Type, M, L, rc, Lambda, PEnergy, &
                            Virial, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, Lambda
    real(8), intent(out) :: PEnergy, Virial
    real(8), parameter :: k = 3000., r0 = 1
    integer :: i,j !Local
    real(8) :: d, d2, id2, id4, id6, id12, rc2, Half_k, Shift, RepelShift, Typei
    real(8), dimension(Dim) :: rij, Posi

    PEnergy = 0.0
    Virial = 0.0
    rc2 = rc*rc
    Shift = -4.0*(rc**(-12) - rc**(-6))
    RepelShift = -4.0*(rc**(-12) + Lambda*rc**(-4))
    Half_k = 0.5*k

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1

            rij = Pos(j,:) - Posi

            !Convert to minimum image distance for periodic boundaries
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
            !Compute the energy and force depending on whether the atoms are 
            !bonded or not.
            if (j == i+1 .and. mod(j,M) > 0) then !bonded
                d = sqrt(d2)
                Virial = Virial + k*d*(d - r0)
                PEnergy = PEnergy + Half_k*(d - r0)*(d - r0)
            !check to see if we are within the cutuff distance
            elseif (d2 < rc2) then !not bonded
                if ((Typei - Type(j)) == 0) then !same type of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id6 = id2*id2*id2    !inverse sixth distance
                    id12 = id6*id6         !inverse twelfth distance
                    Virial = Virial + (-48.*id12 + 24.*id6)
                    PEnergy = PEnergy + 4.*(id12 - id6) + Shift
                else !different kind of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id4 = id2*id2    !inverse fourth distance
                    id12 = id4*id4*id4        !inverse twelfth distance
                    Virial = Virial + (-48.*id12 - 16.*Lambda*id4)
                    PEnergy = PEnergy + 4.*(id12 + Lambda*id4) + RepelShift
                endif ! atom type test
            endif ! bonded test
        enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos, Type, M, L, rc, Lambda, PEnergy, &
                            Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, Lambda
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    integer :: i,j !Local
    real(8) :: d, d2, id2, id4, id6, id12, rc2, Half_k, Shift, RepelShift, Typei
    real(8), dimension(Dim) :: rij, Fij, Posi

    !compute shift energy for cutoff distance of rc
    PEnergy = 0.0
    Forces = 0.0
    rc2 = rc*rc    
    Shift = -4.0*(rc**(-12) - rc**(-6))
    RepelShift = -4.0*(rc**(-12) + Lambda*rc**(-4))
    Half_k = 0.5*k

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1

            rij = Pos(j,:) - Posi
            !Convert to minimum image distance for periodic boundaries
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)

            !Compute the energy and force depending on whether the atoms are 
            !bonded or not.
            if (j == i+1 .and. mod(j,M) > 0) then !bonded
                d = sqrt(d2)
                Fij = rij*k*(1. - r0/d)
                PEnergy = PEnergy + Half_k*(d - r0)*(d - r0)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            !check to see if we are within the cutuff distance
            elseif (d2 < rc2) then !not bonded
                if ((Typei - Type(j)) == 0) then !same type of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id6 = id2*id2*id2    !inverse sixth distance
                    id12 = id6*id6         !inverse twelfth distance
                    Fij = rij*((-48.*id12 + 24.*id6)*id2)
                    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij
                    PEnergy = PEnergy + 4.*(id12 - id6) + Shift
                else !different kind of atom
                    id2 = 1./d2            !inverse s   quared distance
                    id4 = id2*id2    !inverse fourth distance
                    id12 = id4*id4*id4        !inverse twelfth distance
                    Fij = rij*id2*(-48.*id12 - 16.*Lambda*id4)
                    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij
                    PEnergy = PEnergy + 4.*(id12 + Lambda*id4) + RepelShift
                endif ! atom type test
            endif ! bonded test
        enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, Type, M, L, rc, Lambda, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in) :: L, rc, dt, Lambda
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Type, M, L, rc, Lambda, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine


subroutine Structure(Pos, Type, M, L, Lambda, CN, &
                        cRDF, cRDFAA, cRDFAB, Dim, NAtom, Bins)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, Bins
    integer, intent(in), dimension(0:NAtom-1) :: Type
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(inout), dimension(0:NAtom-1) :: CN!,CNAA,CNBB,CNAB ! Coordination Number
!f2py intent(in,out) :: CN
    real(8), intent(in) :: L, Lambda
    real(8), intent(inout), dimension(0:Bins-1) :: cRDF,cRDFAA,cRDFAB !Counts in each bin
!f2py intent(in,out) :: cRDF,cRDFAA,cRDFAB
    integer :: i,j,bin !Local
    real(8), parameter :: rb = 1.25 !bonded distance
    real(8) :: d, Typei, binwidth
    real(8), dimension(Dim) :: rij, Posi

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        Typei = Type(i)
        do j = i + 1, NAtom - 1

            rij = Pos(j,:) - Posi
            !Convert to minimum image distance for periodic boundaries
            rij = rij - L*dnint(rij/L)
            d = sqrt(sum(rij*rij))
            binwidth = L/2.0/Bins
            bin = int(d/binwidth)
            if (bin < Bins) then
                if ((Typei - Type(j)) == 0) then !same type of atom
                    cRDFAA(bin) = cRDFAA(bin) + 2
!                    if (d < rb) CNAA((/i,j/)) = CNAA((/i,j/)) + 1
                else !different type of atom
                    cRDFAB(bin) = cRDFAB(bin) + 2
!                    if (d < rb) CNAB((/i,j/)) = CNAB((/i,j/)) + 1
                endif ! atom type test
                cRDF(bin) = cRDF(bin) + 2
                if (d < rb) CN((/i,j/)) = CN((/i,j/)) + 1
            endif
        enddo
    enddo
end subroutine